indexing
	description: "[
			A graph whose nodes contain data of type N.
			]"
	instructions: "[
			Be sure to redefine `new_edge' in order to create edges of the 
			desired	type.
			]" 
	author: "Jimmy J. Johnson"
	date: "3 Mar 03"

class
	VALUED_GRAPH [N]
	
inherit
	
	SIMPLE_GRAPH
		redefine
			root,
			path,
			new_edge
		end
	
create
	default_create
	
feature -- Access

	root: VALUED_NODE [N]
			-- Node at which to start traversals.
			-- Anchor for node types.

	path: VALUED_PATH [N] is
			-- Clone of path traversed to get to current node.
			-- Anchor for path types. 
			-- (Renamed `item' from LINKED_LIST.)
		do
			Result ?= Precursor {SIMPLE_GRAPH}
		end
		
	new_edge: VALUED_EDGE [N] is
			-- Creates a new edge and also makes the Result available
			-- in `last_new_edge'.  There are no nodes with this edge.
			-- Anchor for edge types.
			-- Redefine to create a different type of edge.
		do
			create Result
			last_new_edge := Result
		end

feature -- Element change

	set_root_with_value (a_value: N) is
			-- Set `root' to the node containing `a_value'.
		require
			value_exists: a_value /= Void
			has_valued_node: find_valued_node (a_value) /= Void
		do
			set_root (find_valued_node (a_value))
		ensure
			root_was_set_with_value: equal (root.value, a_value)
		end

	connect_with_values (a_value_1, a_value_2: N) is
			-- Find the first two nodes that have values `a_value_1' and
			-- `a_value_2', (or create new nodes if they are not in the
			-- graph) connecting them with a new edge.  The new edge can 
			-- be accessed through `last_new_edge'.
		require
			value_1_exists: a_value_1 /= Void
			value_2_exists: a_value_2 /= Void
		local
			n1, n2: like root
		do
			n1 := find_valued_node (a_value_1)
			if n1 = Void then
				create n1.make (a_value_1)
			end
			n2 := find_valued_node (a_value_2)
			if n2 = Void then
				create n2.make (a_value_2)
			end
			connect (n1, n2)
--			is_sorted := False
		ensure
			value_1_node_in_graph: find_valued_node (a_value_1) /= Void
			value_2_node_in_graph: find_valued_node (a_value_2) /= Void
			new_edge_in_graph: edges.has (last_new_edge)
			proper_from_connection_made: last_new_edge.originates_at (find_valued_node (a_value_1))
			proper_to_connection_made: last_new_edge.terminates_at (find_valued_node (a_value_2))
		end

feature -- Basic operations

	find_valued_node (a_value: N): like root is
			-- The first node with `value' the same as `a_value' (object or
			-- reference equality) that is "visible" (i.e. in `nodes') to
			-- the graph.  The resulting node was not necessarily visited
			-- during the last sort.
			-- This is simply a convinence feature for serching `nodes'.
		require
			value_exists: a_value /= Void
		do
			from nodes.start
			until nodes.exhausted or Result /= Void
			loop
--				if nodes.object_comparison then
					if a_value.is_equal (nodes.item.value) then
						Result := nodes.item
					end
--				else
--					if a_value = nodes.item.value then
--						Result := nodes.item
--					end
--				end
				nodes.forth
			end
		end
		
end -- class VALUED_GRAPH [N]
