indexing
	description: "[
			Holds values as nodes for use in WEIGHTED_GRAPHs.  The `value' 
			field of the nodes will be type N.
			]"
	instructions: "[
			It can be used "as is" as a generic container or 'edge' can
			be redefined to change the type of the edges.
			]" 
	author: "Jimmy J. Johnson"
	date: "25 Feb 03"

class
	VALUED_NODE [N]

inherit
	
	SIMPLE_NODE
		redefine
			infix "<"
		end

create
	make
	
feature {NONE} -- Initialization

	make (a_value: N) is
			-- 
		require
			value_exists: a_value /= Void
		do
			default_create
			value := a_value
		end
		
feature -- Access
	
	value: N
			-- The data stored in this node.
	
feature -- Element change

	set_value (a_value: like value) is
			-- Change `value' to `a_value'
			-- Can be Void
		do
			value := a_value
		end

feature -- Comparison

	infix "<" (other: like Current): BOOLEAN is
			-- Is current object less than `other'?
			-- Checks the `value' features of each node.
		local
			c, other_c: COMPARABLE
		do
			if Current = other then
					-- They are the same node (reference equality)
					-- Defaults to False.
			else
				if equal (value, other.value) then
						-- Both `value' features are equal or both Void
						-- so compare the number of edges in each node.
					Result := Precursor {SIMPLE_NODE} (other)
				else
					check
						value_not_void: value /= Void
						other_value_not_void: other.value /= Void
							-- because if both were void would have gone
							-- to precursor in above "if" statement.
					end
						-- Values are not equal and neither is Void, so compare
						-- values, but only if both values are COMPARABLEs.
					c ?= value
					other_c ?= other.value
					if c /= Void and other_c /= Void then
							-- The `value' stored in the node and the `value' stored
							-- in the `other' node are both COMPARABLEs and therefore
							-- can be compared with `<'.
						Result := c < other_c
					end
				end
			end
		end

invariant
	
	value_exists: value /= Void
	
end -- class VALUED_NODE [N]
