indexing
	description: "[
			An edge represents a connection between two SIMPLE_NODEs,
			usually	used in a GRAPH.
			Adds a "cost" feature to SIMPLE_EDGE.
			]"
	instructions: "[
			]" 
	author: "Jimmy J. Johnson"
	date: "15 Feb 03"

class
	WEIGHTED_EDGE [E]

inherit
	
	SIMPLE_EDGE
		redefine
			check_nodes
		end

creation
	default_create,
	make
	
feature {NONE} -- Initialization

	make (a_cost: E) is
			-- Create an edge holding `a_cost'
		require
			cost_exists: a_cost /= Void
		do
			cost := a_cost
		ensure
			cost_was_set: cost = a_cost
		end
		
feature
	
	cost: E
			-- The price of traversing this edge.
	
feature -- Element change

	set_cost (a_cost: E) is
			-- Change the cost of traversing this edge.
		require
			cost_exists: a_cost /= Void
		do
			cost := a_cost
		ensure
			cost_was_set: cost = a_cost
		end

feature -- comparison

	check_nodes (other: like Current): BOOLEAN is
				-- Redefined from `check_nodes' from SIMPLE_EDGE to check the `cost'
				-- feature of both edges before checking the nodes.
				-- From SIMPLE_EDGE:
				-- "When checking the nodes (`node_from' and then `node_to') of
				-- Current and `other', is the nodes of Current less than
				-- the nodes of `other'?
				-- Used by `<'."
--??? Can I compare a SIMPLE_EDGE and a WEIGHTED_EDGE???
		local
			c, other_c: COMPARABLE
		do
			if equal (cost, other.cost) then
					-- Both `cost' features are equal or both Void,
					-- so compare the nodes at each end by calling precursor.
				Result := Precursor {SIMPLE_EDGE} (other)
			elseif cost /= Void and other.cost /= Void then
					-- costs are not equal and neither is Void, so compare
					-- costs, but only if both costs are COMPARABLEs); 
					-- otherwise compare the `time_stamp' feature of each node.
				c ?= cost
				other_c ?= other.cost
				if c /= Void and other_c /= Void then
						-- The `cost' stored in the edge and the `cost' stored
						-- in the `other' edge are both COMPARABLEs and therefore
						-- can be compared with `<'.
					Result := c < other_c
				end
			elseif cost = Void and other.cost /= Void then
					-- costs are not equal and only `cost' of Current is Void
				Result := True
			else
					-- costs are not equal and only `cost' of `other' is Void.
					-- Defaults to False.
			end					
		end
		
invariant
	
--	cost_exists: cost /= Void
		-- The `cost' must be allowed to be Void, especially for features `connect'
		-- and `new_edge' from the GRAPH classes.

end -- class WEIGHTED_EDGE [E]
