indexing
	description: "[
			Graph whose edges contain data of type E.
			]"
	instructions: "[
			Be sure to redefine 'new_edge' in order to create edges of the desired
			type.
			]" 
	author: "Jimmy J. Johnson"
	date: "2 Mar 03"

class 
	WEIGHTED_GRAPH [E]

inherit
	
	SIMPLE_GRAPH
		redefine
			root,
			path,
			new_edge
		end

creation
	default_create

feature -- Initialization

feature -- Access

	root: WEIGHTED_NODE [E]
			-- Node at which to start traversals.
			-- Anchor for node types.

	path: WEIGHTED_PATH [E] is
			-- Clone of path traversed to get to current node.
			-- Anchor for path types. 
			-- (Renamed `item' from LINKED_LIST.)
		do
			Result ?= Precursor {SIMPLE_GRAPH}
		end
		
	new_edge: WEIGHTED_EDGE [E] is
			-- Creates a new edge and also makes the Result available
			-- in `last_new_edge'.  There are no nodes with this edge.
			-- Anchor for edge types.
			-- Redefine to create a different type of edge.
		do
			create Result
			last_new_edge := Result
		end
		
feature -- Element change

	connect_weighted (a_node, a_other_node: like root; a_cost: E) is
			-- Make sure graph contains the two nodes and connect them
			-- by createing an edge with `a_cost'.  The new edge can be 
			-- accessed through `last_new_edge'.
			-- Does not put descendent nodes into the graph.
		require
			node_exists: a_node /= Void
			other_exists: a_other_node /= Void
			cost_exists: a_cost /= Void
		do
			connect (a_node, a_other_node)
			last_new_edge.set_cost (a_cost)
		ensure
			node_inserted: nodes.has (a_node)
			other_node_inserted: nodes.has (a_other_node)
			edge_inserted: edges.has (last_new_edge)
			proper_from_connection_made: last_new_edge.originates_at (a_node)
			proper_to_connection_made: last_new_edge.terminates_at (a_other_node)
			cost_was_set: last_new_edge.cost = a_cost
		end

feature -- Basic operations

	find_weighted_edge (a_cost: E): like new_edge is
			-- The first edge with `cost' the same as `a_cost' (object or
			-- reference equality) that is "visible" (i.e. in `edges') to
			-- the graph.  The resulting edge was not necessarily traversed
			-- during the last sort.
			-- This is simply a convinence feature for serching `edges'.
		require
			cost_exists: a_cost /= Void
		do
			from edges.start
			until edges.exhausted or Result /= Void
			loop
				if edges.object_comparison then
					if a_cost.is_equal (edges.item.cost) then
						Result := edges.item
					end
				else
					if a_cost = edges.item.cost then
						Result := edges.item
					end
				end
				edges.forth
			end
		end
		
end -- class WEIGHTED_GRAPH [E]
