indexing
	description: "[
			A graph whose nodes contain data of type N and whose edges contain
			data of type E.
			]"
	instructions: "[
			Be sure to redefine 'new_edge' in order to create edges of the desired
			type.
			]" 
	author: "Jimmy J. Johnson"
	date: "3 Mar 03"

class
	WEIGHTED_VALUED_GRAPH [N, E]

inherit
	
	WEIGHTED_GRAPH [E]
		export
			{NONE}
				connect
		redefine
			root,
			path,
			new_edge
		end

	VALUED_GRAPH [N]
		export
			{NONE}
				connect,
				connect_with_values
		redefine
			root,
			path,
			new_edge
		end

feature -- Access

	root: WEIGHTED_VALUED_NODE [N, E]
			-- Node at which to start traversals.
			-- Anchor for node types.

	path: WEIGHTED_VALUED_PATH [N, E] is
			-- Clone of path traversed to get to current node.
			-- Anchor for path types. 
			-- (Renamed `item' from LINKED_LIST.)
		do
			Result ?= Precursor {VALUED_GRAPH}	-- picked one
		end
		
	new_edge: WEIGHTED_VALUED_EDGE [N, E] is
			-- Creates a new edge and also makes the Result available
			-- in `last_new_edge'.  There are no nodes with this edge.
			-- Anchor for edge types.
			-- Redefine to create a different type of edge.
		do
			create Result
			last_new_edge := Result
		end

feature -- Element change

	connect_weighted_with_values (a_value_1, a_value_2: N; a_cost: E) is
			-- Find the first two nodes that have values `a_value_1' and
			-- `a_value_2', (or create new nodes if they are not in the
			-- graph) connecting them with a new edge having `a_cost'. 
			-- The new edge can be accessed through `last_new_edge'.
		require
			value_1_exists: a_value_1 /= Void
			value_2_exists: a_value_2 /= Void
			cost_exists: a_cost /= Void
		do
			connect_with_values (a_value_1, a_value_2)
			last_new_edge.set_cost (a_cost)
		ensure
			value_1_node_in_graph: find_valued_node (a_value_1) /= Void
			value_2_node_in_graph: find_valued_node (a_value_2) /= Void
			new_edge_in_graph: edges.has (last_new_edge)
			proper_from_connection_made: last_new_edge.originates_at (find_valued_node (a_value_1))
			proper_to_connection_made: last_new_edge.terminates_at (find_valued_node (a_value_2))
			cost_was_set: last_new_edge.cost = a_cost
		end

end -- class WEIGHTED_VALUED_GRAPH
